import Glyph.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.awt.event.*;

/**
  * BallGlyph2D implements a two-dimensional Ball-glyph with three
  * properties: width, height and color.
  *
  */

public class BallGlyph2D extends Glyph2D
{
  int width, height;
  int widmin, widmax, heimin, heimax;
  double widmind, widmaxd;
  double heimind, heimaxd;
  int widthattr, heightattr, colorattr;
  Color basecolor;
  double colordata, colmaxd, colmind;

  BallGlyph2D()
    {
      // Set default values for glyph size
      widmin = 1;
      heimin = 1;
      widmax = 20;
      heimax = 20;
      widthattr = heightattr = colorattr = 0;
      basecolor = Color.red; // RED as base color.
    }

  public void setBaseColor(Color c)
    {
      basecolor = c;
    }

	public void draw(int x, int y, Graphics g)
	{  
	  int r = (int)((double)basecolor.getRed()*(colordata-colmind)/(colmaxd-colmind));
	  int gr = (int)((double)basecolor.getGreen()*(colordata-colmind)/(colmaxd-colmind));
	  int b = (int)((double)basecolor.getBlue()*(colordata-colmind)/(colmaxd-colmind));
	  if(r > 255) r = 255;
	  if(gr > 255) gr = 255;
	  if(b > 255) b = 255;
	  g.setColor(new Color(r, gr, b));
	  g.drawOval(x-width/2, y-height/2, width, height);
	}

	int getNumberofProperties()
	{
		return 3;
	}

	String getPropertyName(int ix)
	{
		if(ix == 0) return new String("Width");
		if(ix == 1) return new String("Height");
		if(ix == 2) return new String("Color");
		return new String();
	}

	Component getPropertyComponent(int ix, GlyphControlActionListener gc)
	{
		if(ix == 0) return new DimNumberComponent(this, 0);
		if(ix == 1) return new DimNumberComponent(this, 1);
		if(ix == 2) return new ColorChooserComponent(this, 2, gc);
		return null;
	}

	Image getGlyphImage()
	{
		Image i = new BufferedImage(50, 50, java.awt.image.BufferedImage.TYPE_BYTE_BINARY);
		draw(25, 25, i.getGraphics());
		return i;
	}

  void setPropertyData(int ix, int data)
    {
      if(ix == 0) width = (int)(((double)data-widmind)/(widmaxd-widmind));
      if(ix == 1) height = (int)(((double)data-heimind)/(heimaxd-heimind));
      if(ix == 2) colordata = (double)data;
    }

  void setPropertyData(int ix, double data)
    {
      if(ix == 0) 
	width = (int)(((data-widmind)/(widmaxd-widmind))*(double)(widmax-widmin))+widmin;
      if(ix == 1) 
	height = (int)(((data-heimind)/(heimaxd-heimind))*(double)(heimax-heimin))+heimin;
      if(ix == 2) 
	colordata = data;
    }

  void setPropertyMin(int ix, int data)
    {
      if(ix == 0) widmind = data;
      if(ix == 1) heimind = data;
      if(ix == 2) colmind = (double)data;
    }

  void setPropertyMin(int ix, double data)
    {
      if(ix == 0) widmind = data;
      if(ix == 1) heimind = data;
      if(ix == 2) colmind = data;
    }

  void setPropertyMax(int ix, int data)
    {
      if(ix == 0) widmaxd = data;
      if(ix == 1) heimaxd = data;
      if(ix == 2) colmaxd = (double)data;
    }

  void setPropertyMax(int ix, double data)
    {
      if(ix == 0) widmaxd = data;
      if(ix == 1) heimaxd = data;
      if(ix == 2) colmaxd = data;
    }

  void setWidthMin(int wm) { widmin = wm; }
  void setWidthMax(int wm) { widmax = wm; }
  void setHeightMin(int hm) { heimin = hm; }
  void setHeightMax(int hm) { heimax = hm; }

  void setPropertyDataAttribute(int ix, int dataattr)
    {
      if(ix==0) widthattr = dataattr;
      if(ix==1) heightattr = dataattr;
      if(ix==2) colorattr = dataattr;
    }

  int getPropertyDataAttribute(int ix)
    {
      if(ix==0) return widthattr;
      if(ix==1) return heightattr;
      if(ix==2) return colorattr;
      return -1;
    }
}

/**
  * DimNumberComponent class implements a GUI component that is used to control
  * the Width and Height parameters of BallGlyph2D. 
  */

class DimNumberComponent extends JPanel implements GlyphPropertyControl
{
  BallGlyph2D glyph;
  JTextField minf, maxf;
  int attrnum;

  DimNumberComponent(BallGlyph2D g, int an)
    {
      glyph = g;
      attrnum = an;

      setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
      minf = new JTextField();
      maxf = new JTextField();
      add(minf);
      add(maxf);
    }

  public void setDataMin(double min)
    { 
      minf.setText((new Double(min)).toString());
    }
  public void setDataMin(int min){}
  public void setDataMax(double max){}
  public void setDataMax(int max){}
}

class ColorButtonListener implements ActionListener
{
  BallGlyph2D glyph;
  Color color;
  GlyphControlActionListener gcal;

  ColorButtonListener(BallGlyph2D g, Color c, GlyphControlActionListener gc)
    {
      glyph = g;
      color = c;
      gcal = gc;
    }

  public void actionPerformed(ActionEvent e)
    {
      glyph.setBaseColor(color);
      gcal.refreshDataView(); // Refresh the data view with the new color
    }
}

/** ColorChooserComponent is a component for the user to select the base color for
  * BallGlyph2D. */

class ColorChooserComponent extends JPanel 
  implements GlyphPropertyControl
{
  BallGlyph2D glyph;
  JButton green, blue, red, orange, pink, yellow, cyan, magenta;
  int attrnum;
  GlyphControlActionListener gcal;

  ColorChooserComponent(BallGlyph2D g, int an, GlyphControlActionListener gc)
    {
      gcal = gc;
      glyph = g;
      attrnum = an;

      setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
      addColorButton(Color.green);
      addColorButton(Color.red);
      addColorButton(new Color(100, 100, 255));
      addColorButton(Color.yellow);
      addColorButton(Color.pink);
    }

  void addColorButton(Color c)
    {
      JButton b = new JButton();
      b.setBackground(c);
      b.addActionListener(new ColorButtonListener(glyph, c, gcal));
      add(b);
    }

  public void setDataMin(double min)
    {
    }
  public void setDataMin(int min){}
  public void setDataMax(double max){}
  public void setDataMax(int max){}

}
