import java.awt.*;
import java.lang.*;
import java.awt.event.*;
import java.io.File;
import javax.swing.*;
import javax.swing.border.*;

/**
  * Column implements one column in tabular data. 
  * It constains the data, name, measurement, index and class values of column.
  * @author TL
  */      
public class Column{
   private String name;
   private String measurement;
   private int colType;
   private int rowCount;
   private int[]  intArr;
   private long[] longArr;
   private double[] doubleArr;
   private long minLong;
   private long maxLong;
   private double minDouble;
   private double maxDouble;
   private int classCount; //number of classes of column
   private String[]  classArr; //array of names of classes
   private Index Ind;
  /** Constructor creates array for data of the column.
    * @param attrType -  Attribute type 0= long, 1=double, 2=class
    * @param rowCnt - The number of datarows.
    */
   public Column(int attrType, int rowCnt) {
     minLong = maxLong = 0;
     minDouble = maxDouble = 0;
     this.rowCount = rowCnt;
     this.colType = attrType;
      if (attrType == 0){//long
          longArr = new long[rowCount];
      }
      else
      if (attrType == 1){//double
          doubleArr = new double[rowCount];
      }
      else
      if (attrType == 2){//int
          intArr = new int[rowCount];
      }
   }
  /** Method returns index on the column. It creates it if needed.
    * @return Method returns index of the column.
    * @param attrNo - The number of current attribute (column).
    */
   public Index getIndex(int attrNo) {
      if (Ind == null)
      {
      if (this.colType == 0) //long
         Ind = new Index(longArr,rowCount);
      else
      if (this.colType == 1) //double
         Ind = new Index(doubleArr,rowCount);
      else
      if (this.colType == 2) //class
         Ind = new Index(intArr,rowCount);
      }
      return Ind;
   }
  /** Method creates array for class values.
    * @param cCount - The number of class values. 
    */
   public void initializeClasses(int cCount) {
      this.classCount = cCount; 
      classArr =  new String[this.classCount];
   }
  /** Method returns the number of the class values.
    * @return The number of class values.
    */
   public int getNumberofClasses() {return this.classCount;}
  /** Method is used to setting the value of a class value.
    * @param classNo - The number of the class value. 
    * @param value - The corresponding text. 
    */
   public void setValueofClass(int classNo,String value) {classArr[classNo] = value;}
  /** Method returns the value of a class value.
    * @return The value (string) of given classvalue.
    * @param classNo - The number of the class value.
    */
   public String getValueofClass(int classNo) {return classArr[classNo];}
  /** Method sets the name of the column.
    * @param value - The name of the column.
    */
   public void setName(String value) {name = value;}
  /** Method returns the name of the column.
    * @return The name of the column.
    */
   public String getName() {return this.name;}
  /** Method sets the measurement of the column.
    * @param value - The name of the measurement of the column.
    */
   public void setMeasurement(String value) {this.measurement = value;}
  /** Method returns the measurement of the column.
    * @return The name of the measurement of the column.
    */
   public String getMeasurement() {return this.measurement;}
  /** Method returns the type of the column. 0 - long, 1 - double, 2 - class (int).
    * @return The type of the column. 0 - long, 1 - double, 2 - class (int).
    */
   public int getColumnType() {return colType;}
  /** Sets the value of attribute on given row. 
    * @param rowNumber - The number of the row value is added.
    * @param value - The value to be added.
    */
   public void setLong(int rowNumber, long value) {
     longArr[rowNumber] = value;
     //setting the max and min values
     if (rowNumber == 0)
        minLong = maxLong = value;
     else
     if (value < minLong)
        minLong = value;
     else
     if (value > maxLong)
        maxLong = value; 
   }
  /** Sets the value of attribute on given row.
    * @param rowNumber - The number of the row value is added.
    * @param value - The value to be added.
    */
    public void setDouble(int rowNumber, double value) {
     doubleArr[rowNumber] = value;
     if (rowNumber == 0)
        minDouble = maxDouble = value;
     else
     if (value < minDouble)
        minDouble = value;
     else
     if (value > maxDouble)
        maxDouble = value;
   }
  //following methods are for fetching the data
  /** Sets the value of attribute on given row.
    * @param rowNumber - The number of the row value is added.
    * @param value - The value to be added.
    */
   public void setClass(int rowNumber, int value) {intArr[rowNumber] = value;}
  /** Returns the value of given row.
    * @return The value of given row.
    * @param rowNumber - The number of the row value is added.
    */
   public long getLong(int rowNumber) {
      if(colType == 0) return longArr[rowNumber];
      if(colType == 1) return (long)doubleArr[rowNumber];
      if(colType == 2) return (long)intArr[rowNumber];
      return longArr[rowNumber];
   }
  /** Returns the value of given row.
    * @return The value of given row.
    * @param rowNumber - The number of the row value is added.    
    */
   public double getDouble(int rowNumber) {
      if(colType == 0) return (double)longArr[rowNumber];
      if(colType == 1) return doubleArr[rowNumber];
      if(colType == 2) return (double)intArr[rowNumber];    
      return doubleArr[rowNumber];
   }
  /** Returns the value of given row.
    * @return The value of given row.
    * @param rowNumber - The number of the row value is added.    
    */
   public int getClass(int rowNumber) {return intArr[rowNumber];}

  /** 
    * Returns the minimum value of attribute.
    * @return The minimum value of attribute (column).
    */
   public long getMinLong() {return minLong;} 
  /**
    * Returns the maximum value of attribute.
    * @return The maximum value of attribute (column).
    */
   public long getMaxLong() {return maxLong;}
  /**
    * Returns the minimum value of attribute.
    * @return The minimum value of attribute (column).
    */
   public double getMinDouble() {
    if(colType == 0) return (double)minLong;
    if(colType == 2) return (double)minLong;
    return minDouble;
  }
  /**
    * Returns the maximum value of attribute.
    * @return The maximum value of attribute (column).
    */
   public double getMaxDouble() {
      if(colType == 0) return (double)maxLong;
      if(colType == 2) return (double)maxLong;
      return maxDouble;
   }
}
