import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
  * DataWindowFrame implements DataWindow-class as a container
  * to hold DataView-component and a DataViewControlPanel-component.
  * DataWindowFrame accesses these components through their abstract
  * interfaces and therefore does not depend on their implementation
  * details.
  *
  * @author Jaakko Haakana
  */

public class DataWindowFrame extends JFrame implements ActionListener
{
	DataView dv;
	DataViewControlPanel dvcp;
  JPanel p;
  boolean cpvisible;

  /** 
    * The component constructor creates the window menu and places
    * the dataview and dataviewcontrolpanel -components on a
    * JSplitPane-container so that the user can easily hide
    * the other and use the window space for either component.
    *
    * @param newdv A reference to DataView-component.
    * @param newdvcp A reference to DataViewControlPanel-component.
    */

	DataWindowFrame(String title, DataView newdv, DataViewControlPanel newdvcp)
	{
	  setTitle(title);
		MenuBar bar = new MenuBar();
		Menu filem = new Menu("File");
		filem.add("Open View from File...");
		filem.add("Save View");
		filem.add("Save View As...");
		filem.addSeparator();
		MenuItem miExit = new MenuItem("Close view");
		miExit.setActionCommand("Close");
		miExit.addActionListener(this);
		filem.add(miExit);
		bar.add(filem);
		setMenuBar(bar);

		setSize(new Dimension(440, 500));
		dv = newdv;
		dvcp = newdvcp;
		dvcp.setSize(dvcp.getMinimumSize());

		//	      		Box p = new Box(BoxLayout.Y_AXIS);
				p = new JPanel();/*
		p.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));*/
		//		p.add(dv);
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		p.setLayout(gb);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;
		gb.setConstraints(dv, gbc);
		p.add(dv);

		//		HideButton hb = new HideButton();
		JButton hb = new JButton("Hide/Show control panel");
		hb.addActionListener(this);
		hb.setActionCommand("Hide");
		gbc.gridy = 1;
		gbc.weightx = 0;
		gbc.weighty = 0;
		gb.setConstraints(hb, gbc);
		p.add(hb);

		/*		dvcp.validate();
		dvcp.setSize(dvcp.getMinimumSize());*/

		gbc.gridy = 2;
		gb.setConstraints(dvcp, gbc);
	       		p.add(dvcp);
			cpvisible = true;
			p.validate();

		/*		JSplitPane sp = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		sp.setTopComponent(newdv);
		sp.setBottomComponent(dvcp);
		sp.setOneTouchExpandable(true);
		sp.setDividerSize(10); // 10 pixel divider*/
		setContentPane(p);
		setSize(new Dimension(500, 600));
	}

	public void actionPerformed(ActionEvent e) 
    {
		if (e.getActionCommand() == "Close") {
			setVisible(false);
		}
		if (e.getActionCommand() == "Hide") {
		  if(cpvisible) p.remove(dvcp);
		  else p.add(dvcp);
		  cpvisible = !cpvisible;
		  p.validate();
		}
	}
}

class HideButton extends JComponent
{
  HideButton()
    {
      setMinimumSize(new Dimension(15, 15));
      setMaximumSize(new Dimension(0, 15));
      setPreferredSize(new Dimension(200, 15));
      //      setSize(new Dimension(15, 15));
    }

  public void paint(Graphics g)
    {
      g.setColor(new Color(0, 0, 0));
      g.drawLine(2, getHeight()-1, getHeight()/2+2, 1);
      g.drawLine(getHeight()/2+2, 1, 2+getHeight(), getHeight()-1);
      g.drawLine(2+getHeight(), getHeight()-1, 2, getHeight()-1);
      //      g.fillRect(0, 0, 100, 100);
    }
}
