import java.awt.*;

/**
  * Glyph is an abstract class for all Glyph implementations.
  * It defines an abstract interface which all Glyph classes
  * should implement.
  */

abstract public class Glyph
{
	/** m_scale determines the scale of the Glyph on screen. */
	double m_scale;

	/** Sets the scale of the Glyph. */
	void setScale(double s)
	{
		m_scale = s;
	}

	/** Gets the scale of the Glyph object. */
	double getScale() { return m_scale; }

	/** Gets the number of properties this glyph object implements.*/
	abstract int getNumberofProperties();

	/** Gets the name of a property. @param ix A zero-based index to the list of properties. */
	abstract String getPropertyName(int ix);

	/** Gets the UI-component that implements the GUI for controlling the property's attributes. */
	abstract Component getPropertyComponent(int ix, GlyphControlActionListener gc);

	/** Gets a preview image of the Glyph to be presented on a selection list perhaps. */
	abstract Image getGlyphImage();

	/** Sets the data for Glyph property. */
  abstract void setPropertyData(int ix, int data);

	/** Sets the data for Glyph property. */
  abstract void setPropertyData(int ix, double data);

	/** Sets the minimum value of the data for one property so that the glyph can scale
	* correctly. */
  abstract void setPropertyMin(int ix, int data);
  abstract void setPropertyMin(int ix, double data);

	/* Sets the maximum value of the data for one property so that the glyph can scale correctly */
  abstract void setPropertyMax(int ix, int data);
  abstract void setPropertyMax(int ix, double data);

	/* Sets the data attribute that is bound to a property. */
  abstract void setPropertyDataAttribute(int ix, int dataattr);

	/* Gets the data attribute that is bound to a property. */
  abstract int getPropertyDataAttribute(int ix);
}
