import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

/**
 * ProgramWindow class implements the main window of the program.
 * This window implements the UI for file and dataview opening. 
 * Currently this main window is restricted to handle only
 * tabular data files.
*/

public class ProgramWindow extends JFrame implements ActionListener
{
	TabDataViewFactory dvfac;
  TabDatafile datafile;
  TabDataSet dataset;
  JProgressBar progressBar; 
  JLabel labelTitle,labelLines,labelAttrs;
  Vector vecDataView; // contains opened dataviews
  String filename;

	/** ProgramWindow constructor creates the window menus. The View-menu is
          * filled with a list of implemented DataView-types. This list is
          * given by TabDataViewFactory.
	  * @see TabDataViewFactory
	  * @param fn Filename of the datafile to open in the startup.
	  */

	public ProgramWindow(String fn)
	{
		super("Veto");
	  filename = null;
	  if(fn != null) {
	    filename = fn;
	  }
//		setTitle("Veto");

		addWindowListener(new ProgramWindowAdapter());

		dvfac = new TabDataViewFactory();
		// Generate menus for ProgramWindow
		MenuBar bar = new MenuBar();
		Menu filem = new Menu("File");
		MenuItem mi = new MenuItem("Open Datafile...");
		mi.setActionCommand("DataFile");
		mi.addActionListener(this);
		filem.add(mi);
		filem.addSeparator();
		filem.add("Open Workspace...");
		filem.add("Save Workspace");
		filem.add("Save Workspace As...");
		filem.addSeparator();
		MenuItem miExit = new MenuItem("Exit");
		miExit.setActionCommand("Exit");
		miExit.addActionListener(this);
		filem.add(miExit);
		Menu viewm = new Menu("View");

		// View-menu should really be expanded only when the datafile is opened.
		
		long n = dvfac.getNumberofDataViews();
		for(long i = 0;i<n;i++)
		{
			mi = new MenuItem(dvfac.getDataViewName(i));
			mi.addActionListener(this);
			mi.setActionCommand("View");
			viewm.add(mi);
		}
		viewm.add("New view...");
		viewm.addSeparator();
		MenuItem miClose = new MenuItem("Close all views");
		miClose.setActionCommand("Close");
		miClose.addActionListener(this);
		viewm.add(miClose);
		bar.add(filem);
		bar.add(viewm);

		JPanel labelPane = new JPanel();
		labelPane.setLayout(new BoxLayout(labelPane, BoxLayout.Y_AXIS));

		labelTitle = new JLabel("No data");
		labelLines = new JLabel();
		labelAttrs = new JLabel();
		labelPane.add(labelTitle);
		labelPane.add(labelLines);
		labelPane.add(labelAttrs);
		getContentPane().add(labelPane);

		setSize(new Dimension(400, 300));
		setMenuBar(bar);
		setVisible(true);
		vecDataView = new Vector();

		if(filename != null) 
		{
		  openfile();
		}
	}

	/** actionPerformed is called by the framework when user selects a menu item. */
	public void actionPerformed(ActionEvent e)
	{
 		if(e.getActionCommand() == "View")
		{
			if (datafile == null) {
				actionPerformed(new ActionEvent(this,0,"DataFile",0));
			}
			System.out.println("View");
			DataView dv = dvfac.getDataViewInstance(0);
			DataViewControlPanel dvcp = new TabDataView2DControlPanel(dataset, (TabDataView2D) dv);
			DataWindowFrame dwf = new DataWindowFrame(filename, dv, dvcp);
			dwf.setVisible(true);
			vecDataView.add(dwf);
		}
		if (e.getActionCommand() == "Close") {
			System.out.println("Close");
			for(int i=0; i < vecDataView.size(); ++i)
				((DataWindowFrame) vecDataView.get(i)).dispose();
			vecDataView.removeAllElements();
		}
		if (e.getActionCommand() == "Exit") {
			setVisible(false);
//			try  datafile.close(); } catch (IOException ioe) {}
			System.exit(0);
		}
		if(e.getActionCommand() == "DataFile")
		{
			JFileChooser fc = new JFileChooser();
			ExampleFileFilter filter = new ExampleFileFilter("VF", "Veto Format Files");
			fc.addChoosableFileFilter(filter);
			System.out.println("In");
			if (fc.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
				return;
			System.out.println("Out");
			filename = fc.getSelectedFile().getName();
//			String filename = "/home/fs/jhaakana/java/esim.VF";
//			filename = "esim.VF";
//			System.out.println(filename);
			openfile();
		}
	}

  void openfile()
    {
			datafile = new TabDatafile();
			datafile.setFilename(filename); 
			try {
			  datafile.open();
			} catch(IOException ioe) {
			  System.out.println("Cannot open file.");
			  return;
			}
			dataset = new TabDataSet();
			dataset.setDataSource(datafile);
                        dataset.setProgramWindow(this);
 
			try {			  			
			  dataset.getData();
			} catch(Exception e) {
			  System.out.println("IOException");
			}
			labelTitle.setText("Title: " + datafile.getFileDesc());
			labelLines.setText("Rows : " + datafile.getNumberOfRows());
			labelAttrs.setText("Attrs: " + datafile.getNumberOfCols());
			dvfac.setDataSet(dataset);
			setTitle("VETO - " + filename);
    }

   public void setValue(int value)
   { 
      setTitle("Loading datafile "+String.valueOf(value)+" %"); 
      repaint();
   }
}

class ProgramWindowAdapter extends WindowAdapter {
	public void windowClosing(WindowEvent e) {
//		setVisible(false);
//		try { datafile.close(); } catch(IOException exc) {}
		System.exit(0);
	}
}

