
import java.util.*;

/**
  * TabCursor implements a cursor for tabular data. Cursor can be used
  * to browse through the dataset. 
  * With cursor constraints can be made in data. 
  * Class constraints are not implemented. 
  * @author TL
  */

class TabCursor extends Cursor{
   private Index index;
   private int numberofRows;
   private Vector consVec;
   private int currentRow;
   private int currentIndexRow;
   private TabDataSet dataS;

/**Constrainttypes are used when spcifying the constraints for the data */
   public abstract class constraintType {
	public constraintType(int tagg, int attr) { this.attr = attr; this.tag = tagg;}
	public abstract void clearConstraints();
        public int getAttribute() { return this.attr; }
        public int getConstType() { return this.constType; }
	private int attr;
        public int constType; //0=long, 1=double, 2=int, 3=class
        public int tag;
   }
/**Constrainttypes are used when spcifying the constraints for the data */  
   public class constraintTypeDouble extends constraintType {
   	private double min,max;
       /** Constructor creates constraint with given values.
         * @param tagg - Tag value which is used to specify the constraint.
         * @param attr - The number of the attribute (column).
         * @param min - Minimum value of the constraint.
         * @param max - Maximum value of the constraint. 
         */
	public constraintTypeDouble(int tagg, int attr, double min, double max) {
		super(tagg, attr); this.min=min; this.max=max; constType = 1;
	}
       /** Clears the constraint values. Sets them zero.*/
	public void clearConstraints() {
		min=max=0;
	}
       /** Returns the minimum value of constraint.
         * @return The minimum value of constraint.
         */
	public double getMin() { return min; }
       /** Returns the maximum value of constraint.
         * @return The maximum value of constraint.
         */
	public double getMax() { return max; }
       /** Method is used for testing if some value is against constraint.
         * @return True if value is acceptable.
         * @param value - The tested value
         */
	public boolean isOK(double value) { return (value >= min) &&
						   (value <= max); }
   }
/**Constrainttypes are used when spcifying the constraints for the data */  
   public class constraintTypeInt extends constraintType {
        private int min,max;
       /** Constructor creates constraint with given values.
         * @param tagg - Tag value which is used to specify the constraint.
         * @param attr - The number of the attribute (column).
         * @param min - Minimum value of the constraint.
         * @param max - Maximum value of the constraint.
         */
 	public constraintTypeInt(int tagg, int attr, int min, int max) {
		super(tagg, attr); this.min=min; this.max=max; constType = 2;
	}
       /** Clears the constraint values. Sets them zero.*/
 	public void clearConstraints() { min=max=0; }
       /** Returns the minimum value of constraint.
         * @return The minimum value of constraint.
         */
 	public int getMin() { return this.min; }
       /** Returns the maximum value of constraint.
         * @return The maximum value of constraint.
         */
 	public int getMax() { return this.max; }
       /** Method is used for testing if some value is against constraint.
         * @return True if value is acceptable.
         * @param value - The tested value
         */
 	public boolean isOK(int value) { return (value >= min) &&
					        (value <= max); }
   }
/**Constrainttypes are used when spcifying the constraints for the data */  
   public class constraintTypeLong extends constraintType {
        private long min,max;
       /** Constructor creates constraint with given values.
         * @param tagg - Tag value which is used to specify the constraint.
         * @param attr - The number of the attribute (column).
         * @param min - Minimum value of the constraint.
         * @param max - Maximum value of the constraint.
         */
	public constraintTypeLong(int tagg, int attr, long min, long max) {
		super(tagg, attr); this.min=min; this.max=max;constType = 0;
	}
       /** Clears the constraint values. Sets them zero.*/
 	public void clearConstraints() { min=max=0; }
       /** Returns the minimum value of constraint.
         * @return The minimum value of constraint.
         */
 	public long getMin() { return this.min; }
       /** Returns the maximum value of constraint.
         * @return The maximum value of constraint.
         */
 	public long getMax() { return this.max; }
       /** Method is used for testing if given value is against constraint.
         * @return True if value is acceptable.
         * @param value - The tested value
         */
 	public boolean isOK(long value) { return (value >= min) &&
						 (value <= max); }
   }
/**Constrainttypes are used when spcifying the constraints for the data */  
   public class constraintTypeClass extends constraintType {
   	private Vector classes;
	private boolean include;
       /** Constructor creates constraint with given values.
         * @param tagg - Tag value which is used to specify the constraint.
         * @param attr - The number of the attribute (column).
         * @param classes - Vector which constains classes.
         * @param include - Specifies if given classes are included in constraint.
         */
  	public constraintTypeClass(int tagg, int attr, Vector classes, boolean
				Include) {
		super(tagg, attr); this.include = Include; this.classes=classes; constType = 3;
	}
       /** Clears the constraint values.*/
 	public void clearConstraints() { this.classes.removeAllElements();}
      /** Method is used for testing if given value is against constraint.
        * @return True if value is acceptable.
        * @param classno - Tested class number.
        */
       	public boolean isOK(int classno) { Integer i = new Integer(classno); 
					   return this.include ?
						classes.contains(i) :
						!classes.contains(i); }
    }


  /** Constructor gets tabular dataset, in which it belongs to, as an parameter.  
    * @param The dataset to browse. 
    */
   public TabCursor(TabDataSet dS) {
      dataS = dS;
      numberofRows = dS.getNumberofRows();
	consVec = new Vector();
   }

  /** Adds a constraint to the cursor.  
    * @param tag - Tag specifies the constraint.
    * @param attrNo - The number of attribute.
    * @param Min - The minimum value of constraint.
    * @param Max - The maximum value of constraint.
    */
   public void addConstraint(int tag, int attrNo, long Min, long Max) {
	consVec.add(new constraintTypeLong(tag, attrNo, Min, Max)); 
   }
  /** Adds a constraint to the cursor.
    * @param tag - Tag specifies the constraint.
    * @param attrNo - The number of attribute.
    * @param Min - The minimum value of constraint.
    * @param Max - The maximum value of constraint.
    */
   public void addConstraint(int tag, int attrNo, double Min, double Max){
   	consVec.add(new constraintTypeDouble(tag, attrNo, Min, Max));
   }
  /** Adds a constraint to the cursor.
    * @param tag - Tag specifies the constraint.
    * @param attrNo - The number of attribute.
    * @param class - Vector which constains classes.
    * @param Include - Specifies if given classes are included in constraint.
    */
   public void addConstraint(int tag, int attrNo, Vector Class, boolean Include) {
   	consVec.add(new constraintTypeClass(tag, attrNo, Class, Include));
   }

  /** 
    * Clears the given constraint.  
    * @param Tag specifies which canstraint should be deleted.
    */
   public void clearConstraint(int tag){
      constraintType tmpConstr;
      if (consVec.size() > 0)
         for (int i = consVec.size() - 1; i <= 0; --i) {
             tmpConstr = (constraintType) consVec.elementAt(i);
             if (tmpConstr.tag == tag) {
                consVec.remove(tmpConstr);
                System.out.println("clearing constraint "+tag+" placed in "+i);
             }       
         } 
   }

   private void findNextOKRow()
   {
       currentRow = index.getValue(currentIndexRow);
       if (currentRow != -1) { //some rows were found
          constraintType browsingConst; //this constraint is used when browsing
          boolean OK = false;
          while ((currentRow != -1) && (!OK)) {//searching for the good rows$
             OK = true;
             for (int i = 0; i < consVec.size(); ++i)  {
                browsingConst = (constraintType) consVec.elementAt(i);
                if (browsingConst.getConstType() == 0) { //long
                   constraintTypeLong tmpConst = (constraintTypeLong) browsingConst;
                   OK = OK && tmpConst.isOK(dataS.getLongValue(currentRow,tmpConst.getAttribute())); 
                }
                else           
                if (browsingConst.getConstType() == 1) { //double
                   constraintTypeDouble tmpConst = (constraintTypeDouble) browsingConst;
                   OK = OK && tmpConst.isOK(dataS.getDoubleValue(currentRow,tmpConst.getAttribute()));
                }
                else
                if (browsingConst.getConstType() == 2) {//int
                   constraintTypeLong tmpConst = (constraintTypeLong) browsingConst;
                   OK = OK && tmpConst.isOK(dataS.getLongValue(currentRow,tmpConst.getAttribute()));
                }
                else  
                if (browsingConst.getConstType() == 3); //class !!NOT implemented    
             }//  for loop
             if (!OK) {
                 ++currentIndexRow;
                 currentRow = index.getValue(currentIndexRow);
             }
          }  //while loop
       }  //currentRow!=-1
   }  //findNextOkRow

  /** Moves the cursor on the first row of the data minding the constraints. */
   public void findFirst()  {
      int indexColumn; //indicates which column should have index
      constraintType browsingConst; //this constraint is used when browsing
      if (consVec.size() == 0)
         currentRow = 0;
      else { //have to use an index
           if (consVec.size() == 1)  //only one constraint -> index on it
            browsingConst = (constraintType) consVec.elementAt(0);
         else
             browsingConst = (constraintType) consVec.elementAt(0); 
         //finding the smallest value by using index          
         index = dataS.getIndex(browsingConst.getAttribute());
         if (browsingConst.getConstType() == 0) //long
         {
            constraintTypeLong tmpConst = (constraintTypeLong) browsingConst;
            currentIndexRow = index.findFirst(tmpConst.getMin());
         }
 	 else
         if (browsingConst.getConstType() == 1) //double
         {
            constraintTypeDouble tmpConst = (constraintTypeDouble) browsingConst;
            currentIndexRow = index.findFirst(tmpConst.getMin());
         }
         else      
         if (browsingConst.getConstType() == 2) //int
         {
            constraintTypeInt tmpConst = (constraintTypeInt) browsingConst;
            currentIndexRow = index.findFirst(tmpConst.getMin());
         }
         else 
         if (browsingConst.getConstType() == 3) //class
         {
/*            int smallestClass;
            constraintTypeClass tmpConst = (constraintTypeClass) browsingConst;
            if (tmpConst.classes.size() == 0) //no constrained classes
               smallestClass = 0;
            else
               smallestClass = (int) tmpConst.classes.elementAt(0);
            //now searching the other values
            for (int i = 1; i < tmpConst.classes.size(); ++i)
               if (smallestClass >  (int) tmpConst.classes.elementAt(i))
                  smallestClass =  (int) tmpConst.classes.elementAt(i);  
            currentIndexRow = index.findFirst(smallestClass);*/
         }
         findNextOKRow(); 
      }//   consVec.size() != 0
   }

  /** Move the cursor to the next row of the data minding the constraints. */
   public void moveNext() {
      if (consVec.size() == 0)
         ++currentRow;
      else  {
         ++currentIndexRow;
         findNextOKRow();
      }
   }

  /** EOF (End of File) is used to finding out if current row is valid. 
    * @return False false if cursor is placed on a valid row. 
    *               true if cursor have passed the last valid row.
    */
   public boolean EOF() {return ((currentRow>=numberofRows) || (currentRow < 0))    ;}
  /** Method returns the value of given attribute.
    * @return The value of given attribute.
    * @param attrNo - The number of attribute.
    */
   public long getLongValue(int attrNo){return dataS.getLongValue(currentRow,attrNo);}
  /** Method returns the value of given attribute.
    * @return The value of given attribute.
    * @param attrNo - The number of attribute.
    */
   public double getDoubleValue(int attrNo){return dataS.getDoubleValue(currentRow,attrNo);}
  /** Method returns the value of given attribute.
    * @return The value of given attribute.
    * @param attrNo - The number of attribute.
    */
    public int getClassValue(int attrNo){return dataS.getClassValue(currentRow,attrNo);}
  /** Method returns true, if current row is selected. Not implemented. 
    * @return If the current row is selected method returns true.
    */ 
   public boolean isSelected() {return false;}
}
