import java.io.*;
import java.awt.*;
import java.lang.*;
import java.awt.event.*;
import java.io.File;
import javax.swing.*;
import javax.swing.border.*;
 
/**
  * TabDataSet implements a dataset for tabular data.
  * @author TL
  */

public class TabDataSet extends DataSet implements Constants {
   private int rowCount;
   private int colCount;
   private int attrNumber;
   private String fileName;
   private String Title;
   private Column[] Columns;
//   Attributes

   public TabDataSet(){super();}

  /**
    * Starts to fetch data from the tabular DataSource.
    * @return Returns true if fetching was succesfull.
    * @exception Raises exception- typed exception.
    */

   public boolean getData() throws Exception{
      boolean OK = true;
      TabDataSource tds;
      tds = (TabDataSource)  this.getDataSource();
      colCount = tds.getNumberOfCols();
      rowCount = tds.getNumberOfRows();
      Title = tds.getFileDesc();

      //creating the columns
      Columns = new Column[colCount];
      Column Col;
      Thread tmpThread;
      tmpThread = Thread.currentThread();
      int tmpColNo = 0; 
      for(int i = 0;i < colCount;++i)
      {
          Col = new Column(tds.getColumnType(i),rowCount);
          Columns[i] = Col;
          try {
             Col.setName(tds. getColumnName(i));
             Col.setMeasurement(tds.getColumnMeasurement(i));
             if (tds.getColumnType(i) == 2) { //is a class type
                  Col.initializeClasses(tds.getNumberOfClasses(i));
                  for(int j = 0;j < tds.getNumberOfClasses(i);++j)
                     Col.setValueofClass(j,tds.getValueOfClass(i,j));
             }
          }
          catch (Exception e) {
             if (DEBUG)  { 
                System.out.println (i+1+" column, error on it.");
                System.out.println (e.getMessage());
             }
             throw e;
          }
          ++tmpColNo;  
      }
      if (tmpColNo != colCount){
         if (DEBUG) System.out.println("Warning!: The number of column information differs from the given number.");
         OK = false;
      }
      //fetching the data
      int row = 0;
      int showInt = rowCount / 100;
      if (showInt == 0) 
          showInt = 1;
      if (!tds.fetchFirst())
          OK = false;
      else
           do{
              try {  
               for(int i = 0;i<colCount;++i) {
                     if (Columns[i].getColumnType() == T_INTEGER) //long
                      Columns[i].setLong(row,tds.getColAsLong(i));
                   else
                   if (Columns[i].getColumnType() == T_DOUBLE) //double
                      Columns[i].setDouble(row,tds.getColAsDouble(i));
                   else
                   if (Columns[i].getColumnType() == T_CLASS) //class
                      Columns[i].setClass(row,tds.getColAsClass(i));
               }
              }
              catch (Exception e) {
                 if (DEBUG)  { 
                    System.out.println (row+1+" row, error on it.");
	            System.out.println (e.getMessage());
		}
		throw e;
              }
              if ((row % showInt) == 0)  { //only then progressbar is updated
                  if (rowCount == 0)
                     programWindow.setValue(0);
                  else
                     programWindow.setValue((int)(((float) row) /((float) rowCount) * 100));
                  try { tmpThread.sleep(1); }
                  catch (InterruptedException e) {
                    if (DEBUG)  System.out.println ("Thread interrupted.");
                  }    
              }
              ++row;
           }while(tds.fetchNext() && (row<=rowCount));
      if (row != rowCount){
         if (DEBUG) System.out.println("Warning!: The number of datarows in file differs from the number in header");
         OK = false;
      }
      return OK ;
   }

  /** Method returns TabCursor object for browsing of the data. 
    * @return A TabCursor object for browsing of the data. 
    */
   public TabCursor getCursor() {
      TabDataSet dS;
      dS = this;
      return new TabCursor(dS);
   }

  /** Method gives index for given attribute (column). It also creates it when needed.  
    * @return Index-object for given attribute.  
    */
   public Index getIndex(int attrNo) {return Columns[attrNo].getIndex(attrNo);}

  /** Method returns the value of the given attribute on the given row. 
    * @return The value of the given attribute on the given row. 
    * @param rowNo - The number of row and number of attribute (column).
    * @param attrNo - The number of attribute (column).
    */
   public long getLongValue(int rowNo, int attrNo) {return Columns[attrNo].getLong(rowNo);}
  /** Method returns the value of the given attribute on the given row.
    * @return The value of the given attribute on the given row. 
    * @param rowNo - The number of row and number of attribute (column).
    * @param attrNo - The number of attribute (column).              
    */
   public double getDoubleValue(int rowNo, int attrNo) {return Columns[attrNo].getDouble(rowNo);}
  /** Method returns the value of the given attribute on the given row. 
    * @return The class value of the given attribute on the given row. 
    * @param rowNo - The number of row and number of attribute (column).
    * @param attrNo - The number of attribute (column).              
    */
   public int getClassValue(int rowNo, int attrNo) {return Columns[attrNo].getClass(rowNo);}

  /** Sets some of the rows in the set as selected. Not implemented.*/
   public void setSelectedRows(int Attr1No, long Attr1Min, long Attr1Max, int Attr2No, long Attr2Min, long Attr2Max) {}
  /** Sets some of the rows in the set as selected. Not Implemented.*/
   public void setSelectedRows(int Attr1No, long Attr1Min, long Attr1Max, int Attr2No, double Attr2Min, double Attr2Max) {}
  /** Sets some of the rows in the set as selected. Not implemented. */
   public void setSelectedRows(int Attr1No, double Attr1Min, double Attr1Max, int Attr2No, long Attr2Min, long Attr2Max) {}
  /** Sets some of the rows in the set as selected. Not implemented.*/
   public void setSelectedRows(int Attr1No, double Attr1Min, double Attr1Max, int Attr2No, double Attr2Min, double Attr2Max) {}
  /** Removes the selection from all the selected rows. Not implemented.*/
   public void removeSelectedRows(){}

  /** Method returns the number of rows in the dataset.
    * @return The number of rows in the dataset. 
    */
   public int getNumberofRows() {return rowCount;}
  /** Method returns the number of attributes (columns) in the dataset.  
    * @return The number of attributes (columns) in the dataset. 
    */
   public int getNumberofAttributes() {return colCount;}
  /** Method returns the name of the given attribute (column).
    * @return The name of the given attribute (column). 
    * @param attrNo - Number of the attribute (column). 
    */
   public String getAttributeName(int attrNo) {
      return Columns[attrNo].getName();
   }   
  /** Method returns the measurement of the given attribute.
    * @return The measurement of the given attribute. 
    * @param attrNo - Number of the attribute (column).
    */
   public String getAttributeMeasurement(int attrNo) {
      return Columns[attrNo].getMeasurement();
   }    
  /** Method returns the  type of the given attribute: 0=long, 1=double, 2=class.
    * @return The type of the given attribute: 0=long, 1=double, 2=class. 
    * @param attrNo - Number of the attribute (column).
    */
   public int getAttributeType(int attrNo) {
      return Columns[attrNo].getColumnType();
   } //0 = long, 1= double, 2 = class

  /** Method returns the  number of class values the class typed attribute contains. 
    * @return The number of class values the class typed attribute contains. 
    * @param attrNo - Number of the attribute (column).
    */
   public int getNumberofClasses(int attrNo){
      return Columns[attrNo].getNumberofClasses();
   }
  /** Method returns the  value (string) of the given class of the given attribute.
    * @return The value (string) of the given class of the given attribute.
    * @param attrNo - Number of the attribute (column).
    * @param classNo -  Number of the class.
    */
    public String getValueofClass(int attrNo, int classNo) {
      return Columns[attrNo].getValueofClass(classNo);
   }

  /** Method returns the minimum value of the given attribute. 
    * @return The minimum value of the given attribute. 
    * @param attrNo - Number of the attribute (column).
    */
    public long getMinLong(int attrNo) {
      return Columns[attrNo].getMinLong();
    }
  /** Method returns the maximum value of the given attribute.
    * @return The maximum value of the given attribute. 
    * @param attrNo - Number of the attribute (column).
    */
   public long getMaxLong(int attrNo) {
      return Columns[attrNo].getMaxLong();
   }   
  /** Method returns the minimum value of the given attribute.
    * @return The minimum value of the given attribute.
    * @param attrNo - Number of the attribute (column).
    */
   public double getMinDouble(int attrNo) {
      return Columns[attrNo].getMinDouble();
   }
  /** Method returns the minimum value of the given attribute.
    * @return The minimum value of the given attribute.
    * @param attrNo - Number of the attribute (column).
    */
   public double getMaxDouble(int attrNo) {
      return Columns[attrNo].getMaxDouble();
   } 
}
