import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.lang.Math.*;

/** 
  * This Canvas is the main drawing Canvas for TabDataView2D.
  * It implements the drawing of the buffered image.
  */

class MyCanvas extends Canvas
{
	boolean selection; // Whether user is selecting something with mouse.
	int selstartx, selstarty;
	int selendx, selendy;
  DataView dataview;
  Image dbuf;
  int oldwidth, oldheight;

	MyCanvas(DataView dv)
	{
	  oldwidth = oldheight = 0;
	  dbuf = null;
	  dataview = dv;
		selection = false;
	}

	/** paint method paints the canvas with a buffered image if it is available.
          * and otherwise it starts a drawing thread by calling the DataView's
          * startDraw() -method. */
	public void paint(Graphics g)
	{
	  if(getWidth() != oldwidth || getHeight() != oldheight) {
	    oldwidth = getWidth();
	    oldheight = getHeight();
	    dataview.startDraw();
	  } else
	  if(dbuf != null) {
	    	    g.drawImage(dbuf, 0, 0, this);
	  }
	}

	int min(int a, int b) { if(a<b) return a; else return b; }
	int abs(int a) { if(a<0) return -a; else return a; }

  public void setImage(Image i)
    {
      dbuf = i;
    }
}

/**
  * TabDataView2D provides a two-dimensional view to tabularly formatted
  * data. DataView acts as a component for the drawing canvas and the 
  * Ruler-components which are used to draw the two axis.
  *
  */

public class TabDataView2D extends DataView
  implements ActionListener, ChangeListener
{
  Glyph2D glyph;
  MyCanvas c;
  TabDataSet dataset;
  TabCursor cursor;
  boolean gridon;

  /** Attribute indices for attributes seleted for the axis. */
  int xattr, yattr;

  /** Maximum and maximum values for selected attributes. */
  double maxx, maxy, minx, miny; 

  /** Ruler-components for the axis. */
  Ruler xruler, yruler;
  Image dbuf;
  static DataWindow dwindow;
  JToggleButton butData;
  JToggleButton butGrid;
  JButton butBgColor;
  Color bgcolor;

  /** 
    * TabDataView2D is constructed with a reference to the 
    * dataset-object in use.
    * The component uses GridBagLayout to layout the necessary
    * component 
    */

	TabDataView2D(TabDataSet ds)
	{
	  xattr = yattr = 0;
	  dataset = ds;
	  //	  bgcolor = new Color(120, 120, 120);

	  		JPanel toolbar = new JPanel();
		toolbar.add(new JToggleButton("Sel"));
		toolbar.add(new JToggleButton("Zoom"));
		butData = new JToggleButton("Data");
		butData.setActionCommand("Data");
		butData.addActionListener(this);
		toolbar.add(butData);
		butGrid = new JToggleButton("Grid");
		butGrid.setSelected(true);
		butGrid.addChangeListener(this);
		gridon = true;
		toolbar.add(butGrid);
		/*		butBgColor = new JButton("Background Color");
		butBgColor.addActionListener(this);
		butBgColor.setActionCommand("BGCOLOR");
		toolbar.add(butBgColor);*/

		c = new MyCanvas(this);

		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(gb);

		butData = new JToggleButton("Data");

	       	gbc.fill = GridBagConstraints.BOTH;
		gbc.gridy = 0;
		gbc.weightx = 0;
		gbc.weighty = 0;
		gbc.gridx = 1;
		gb.setConstraints(toolbar, gbc);
		add(toolbar);

		gbc.weighty = 1;
		gbc.weightx = 1;
		gbc.gridx = 1;
		gbc.gridy = 1;
		gb.setConstraints(c, gbc);
		add(c);

		gbc.weightx = 0;
		gbc.weighty = 0;
		gbc.gridx = 0;
		gbc.gridy = 1;
		yruler = new Ruler(false, dataset.getMinDouble(yattr), dataset.getMaxDouble(yattr)); // vertical ruler
		gb.setConstraints(yruler, gbc);
		add(yruler);

		gbc.gridy = 2;
		gbc.gridx = 1; // one empty spot below yruler.
		xruler = new Ruler(true, dataset.getMinDouble(xattr), dataset.getMaxDouble(xattr));
		gb.setConstraints(xruler, gbc);
		add(xruler);
				validate();
		xruler.setMin(dataset.getMinDouble(xattr));
		xruler.setMax(dataset.getMaxDouble(xattr));
		yruler.setMin(dataset.getMinDouble(yattr));
		yruler.setMax(dataset.getMaxDouble(yattr));
	}

  /** drawGrid draws a grid in a view before the glyphs are drawn. Hints for
    * drawing are requested from the Ruler components. */

  void drawGrid(Graphics g)
    {
      g.setColor(new Color(150, 150, 150));
      int i;
      double dmin, dmax, dlen;
      dmin = dataset.getMinDouble(xattr);
      dmax = dataset.getMaxDouble(xattr);
      dlen = dmax-dmin;
      xruler.calculateAll();
	  double s = xruler.getTickStart();
	  double d = xruler.getTickDelta();
      for(i=0;i<xruler.getTickCount();i++)
	{
	  int x = (int)((s-dmin)/(dmax-dmin)*(double)c.getWidth()) + 
	    i*(int)(d/(dmax-dmin)*(double)c.getWidth());
	  g.drawLine(x, 0, x, c.getHeight());
	}
      dmin = dataset.getMinDouble(yattr);
      dmax = dataset.getMaxDouble(yattr);
      dlen = dmax-dmin;
      yruler.calculateAll();
      s = yruler.getTickStart();
      d = yruler.getTickDelta();
      for(i=0;i<yruler.getTickCount();i++)
	{
	  int y = c.getHeight() - ((int)((s-dmin)/(dmax-dmin)*(double)c.getHeight()) + 
			     i*(int)(d/(dmax-dmin)*(double)c.getHeight()));
	  g.drawLine(0, y, c.getWidth(), y);
	}
      
    }

  /** drawOne() draws one glyph in the view using the current data in the cursor.
    */
  
  void drawOne()
    {
      if(c != null && dbuf != null) {
	Graphics g = c.getGraphics();
	Graphics g2 = dbuf.getGraphics();

	if(g != null && g2 != null) 
	  {
	    if(cursor == null) {
			      System.out.println("cursor is null");
			      return;
	    }
       			  double x, y;
       			  x = cursor.getDoubleValue(xattr);
       			  y = cursor.getDoubleValue(yattr);
				if(glyph != null) {
				  for(int i=0 ; i<glyph.getNumberofProperties();i++) {
				    int dix = glyph.getPropertyDataAttribute(i);
				    glyph.setPropertyData(i, cursor.getDoubleValue(dix));
				  }

				  glyph.draw((int)((x-minx)/(maxx-minx)*((double)c.getWidth())), 
c.getHeight()-(int)((y-miny)/(maxy-miny)*((double)c.getHeight())), g);
				  				  glyph.draw((int)((x-minx)/(maxx-minx)*((double)c.getWidth())), 
c.getHeight()-(int)((y-miny)/(maxy-miny)*((double)c.getHeight())), g2);
				} else {
				  System.out.println("glyph is null!");
				}
			}
		}
    }

  /** drawFirst() clears the view, draws the grid, rewinds the cursor and draws the first glyph. */

	public boolean drawFirst()
	{
	  validate(); // validate() is needed so that the axis get redrawn correctly.
	  dbuf = c.createImage(c.getWidth(), c.getHeight());
	  if(dbuf == null) return false;
	  //	  c.setBackground(bgcolor);
	  Graphics g = c.getGraphics();
	  Graphics g2 = c.getGraphics();
	  if(g != null) {
	    g.clearRect(0, 0, c.getWidth(), c.getHeight());
	    g2.clearRect(0, 0, c.getWidth(), c.getHeight());
	    if(gridon) {
	      drawGrid(g);
	      drawGrid(g2);
	    }
	  }
		minx = dataset.getMinDouble(xattr);
		maxx = dataset.getMaxDouble(xattr);
		miny = dataset.getMinDouble(yattr);
		maxy = dataset.getMaxDouble(yattr);
		cursor = dataset.getCursor();
		cursor.findFirst();
		if(!cursor.EOF()) {
		  drawOne();
		  return true;
		}

		return false;
	}

/** drawNext moves the cursor on the next row and draws a glyph using the data in the cursor. */
	
	public boolean drawNext()
	{
	  cursor.moveNext();
	  if(!cursor.EOF()) {
		  drawOne();
		  return true;
	  }

	  return false;
	}

/** drawFinal is called by DataView's drawing thread when all the glyphs are drawn.
  * The buffered image is set in the canvas so that it can be drawn when the window
  * needs refreshing. */

  public boolean drawFinal()
    {
      c.setImage(dbuf);
      return true;
    }

  // These methods set how each axis is bound in the data. The parameter
  // is the index of the attribute in the data which the axis is bound.
  // When new axis has been set the data has to be redrawn.

  public void setXAxis(int ix)
    {
      xattr = ix;
      xruler.setMin(dataset.getMinDouble(ix));
      xruler.setMax(dataset.getMaxDouble(ix));
      startDraw();
    }

public void setYAxis(int ix)
    {
      yattr = ix;
      yruler.setMin(dataset.getMinDouble(ix));
      yruler.setMax(dataset.getMaxDouble(ix));
      startDraw();
    }

  public void setGlyph(Glyph2D g)
    {
      glyph = g;
    }

   public void actionPerformed(ActionEvent ae) {
     /*	if (ae.getActionCommand() == "Data") {
		System.out.println("DataWin");
		if (butData.isSelected()) {
			if (dwindow == null)
				dwindow = new DataWindow();
			dwindow.setVisible(true);
		} else {
			dwindow.setVisible(false);
		}
	}*/
	if(ae.getActionCommand() == "BGCOLOR") {
	 Color nc = JColorChooser.showDialog(this, "Choose color for the background", bgcolor);
	 if(nc.getBlue() != bgcolor.getBlue() || nc.getGreen() != bgcolor.getGreen() || nc.getRed() != bgcolor.getRed()) 
	 {
	   bgcolor = nc;
	   startDraw();
	 }

	}
   }

  /**
    * Java framework calls stateChanged when some of the togglebuttons
    * on this view are toggled. There is only one (working) button so the grid is toggled on/off and
    * the view is redrawn.
    */

  public void stateChanged(ChangeEvent e)
    {
      if(butGrid.isSelected())
	gridon = true;
      else
	gridon = false;
      startDraw();
    }
    
   public TabCursor getTabCursor() { return cursor;} 
}
