
import java.io.*;
import java.util.*;

// package veto.data;

/**
 * Datafile implementation
 * @author TH/JVL
*/
public class TabDatafile extends TabDataSource implements Constants {
      private String fname; // filename of data
      private int noCols, noRows; // # of rows/columns
      private String desc; // title/description of data 
      private long lineno; // last line read by fetch 
      private int hdrlen; // lines in header
      private BufferedReader infile;
      private Vector cur_line; // contains attrs of current line

      private class TabCol {
	String name;
	int type; // integer=0, double==1, class==2
	String measurement;
	int numclasses;
	Vector class_info;
    }
    private Vector attr; // contains attr definitions (TabCol)

/**
 * Sets filename for following open() -method
 * @param Complete filename of datafile
 * @return true if filename set successfully
 */
   public boolean setFilename (String filename) {
	this.fname = filename;
	return true;
    };

/**
 * Opens datafile and reads header
 */
    public void open() throws IOException {
	attr = new Vector ();
	InputStream instream;
	  instream = new FileInputStream (fname);
	  try {
	    String s;
	      infile = new BufferedReader (new InputStreamReader (instream));
	      s = infile.readLine();
	      desc = s.substring(7);
	      s = infile.readLine();
	      noRows = Integer.parseInt (s.substring(11));
	      s = infile.readLine();
	      noCols = Integer.parseInt (s.substring(12));

	    while (!(s = infile.readLine()).equals("")) {
		TabCol t = new TabCol();
		//read attr definitions
		StringTokenizer st = new StringTokenizer(s, ",");
		  if (st.hasMoreTokens())
			t.name = st.nextToken();
		  else
		        continue;
		  if (st.hasMoreTokens()) {
			  t.measurement = st.nextToken();
			  t.measurement = (t.measurement).trim();
		} else
		      continue;
		  if (!st.hasMoreTokens())
			  continue;
		String stype = st.nextToken();
		stype = stype.trim();
		if (stype.equals("integer")) t.type = 0;
		else if (stype.equals("double")) t.type = 1;
		else if (stype.equals("class")) t.type = 2;
		else throw new IOException("Invalid type " + stype + " in header");
		if ((t.type == 2) && st.hasMoreTokens()) {
				t.numclasses = Integer.parseInt(st.nextToken());
				t.class_info = new Vector();
				for(int i=0; i < t.numclasses && st.hasMoreTokens(); ++i)
			      t.class_info.add(st.nextToken());
		    } else
		    	t.numclasses=0;
		  attr.add(t);
	      }

	}
	catch (IOException e) {
//	    if (DEBUG) 
                System.out.println ("File not found");
	    throw e;
	};
    };

/**
 * closes datafile
 */
    public void close() throws IOException {
	infile.close ();
    }

/**
 * @return Description of datafile.
 */
    public String getFileDesc() {
	return this.desc;
    }

/**
 * @return Number of rows in datafile.
 */
    public int getNumberOfRows() {
	return this.noRows;
    }

/**
 * @return Number of columns in datafile.
 */
    public int getNumberOfCols() {
	return this.noCols;
    }

/**
 * @param ix Index to attribute-table
 * @return Name of attribute
 */
    public String getColumnName(int ix) {
	return ((TabCol) this.attr.get(ix)).name;
    }

/**
 * @param ix Index to attribute-table
 * @return Type of attribute (0==integer,1==double,2==class)
 */
    public int getColumnType(int ix) {
	return ((TabCol) this.attr.get(ix)).type;
    }

/**
 * Reads first line (row) from datafile
 */
    public boolean fetchFirst() throws IOException {
	lineno = 1;
	cur_line = new Vector();
	String lin;
	do {
	try {
		lin = infile.readLine().trim();
	} catch (IOException e) { throw new IOException("Virhe luettaessa datarivi 1"); }
	} while(lin.equals(""));
	if (lin == null)
		throw new IOException("Aineistossa ei datarivej");
	StringTokenizer st = new StringTokenizer (lin, ",");
	while (st.hasMoreTokens ())
	  {
		cur_line.add (st.nextToken().trim());
	  }
	return true;
    }

/**
 * Reads next line (row) from datafile.
 */
    public boolean fetchNext() throws IOException {
	String lin;
	do {
	try {
		lin = infile.readLine();
	} catch(IOException e) { throw new IOException("Error while reading line " + lineno); }
	} while ((lin != null) && (lin.trim().equals("")));	
	if (lin == null) {
		if (lineno != noRows)
			throw new IOException("Rowcount in header incorrect");
		return false;
	}
	StringTokenizer st = new StringTokenizer (lin, ",");
	cur_line = new Vector();
	while (st.hasMoreTokens ()) {
		String s = (st.nextToken()).trim();
		if (s.equals(""))
			throw new IOException("Missing attribute on line " + lineno);
		cur_line.add (s);
	}
	++lineno;
	return true;
    }

/**
 * @param ix Index to attribute-table
 * @return Unit of measurement of attribute (if available)
 */
    public String getColumnMeasurement(int ix) {
    	return ((TabCol) attr.get(ix)).measurement;
    }

/**
 * @param ix Index to attribute-table
 * @return Number of classes in classified attribute (0==not classified)
 */
    public int getNumberOfClasses(int ix) {
    	return ((TabCol) attr.get(ix)).class_info.size();
    }

/**
 * @param ix Index to attribute-table.
 * @param n Index to to list of class-descriptions in attribute-table
 * @return Name of class in a classified-attribute
 */

    public String getValueOfClass(int ix, int n) {
    	return (String) ((TabCol) this.attr.get(ix)).class_info.get(n);
    }

/**
 * @param ix Index to attribute-table
 * @return Value of class-attribute as String
 */
    public String getColAsClassString(int ix) {
	return (String) ((TabCol)
	this.attr.get(ix)).class_info.get(getColAsInt(ix));
}

/**
 * @param ix Index to attribute-table
 * @return Value of attribute as in datafile without type-conversion
 */
    public String getColAsString(int ix) {
	return (String) this.cur_line.get(ix);
    }

/**
 * @param ix Index to attribute-table
 * @return Value of attribute as integer
 */
    public int getColAsInt(int ix) {
	return Integer.parseInt ((String) this.cur_line.get (ix));
    }

/**
 * @param ix Index to attribute-table
 * @return Value of attribute as long integer
 */
    public long getColAsLong(int ix) {
	return Integer.parseInt ((String) this.cur_line.get (ix));
    }

/**
 * @param ix Index to attribute-table
 * @return Value of classified attribute as integer
 */
    public int getColAsClass(int ix) {
    	return Integer.parseInt((String) this.cur_line.get(ix));
    }

/**
 * @param ix Index to attribute-table
 * @return Value of attribute as double
 */
    public double getColAsDouble(int ix) {
	return Double.parseDouble ((String) this.cur_line.get (ix));
    }

}
